@@ -11,6 +11,7 @@ import com.fasterxml.jackson.core.*;
 
 import com.fasterxml.jackson.databind.ObjectMapper;
 import com.fasterxml.jackson.databind.deser.std.StdScalarDeserializer;
+import com.fasterxml.jackson.databind.json.JsonMapper;
 import com.fasterxml.jackson.databind.ser.std.StdScalarSerializer;
 import com.fasterxml.jackson.databind.type.TypeFactory;
 
@@ -208,28 +209,37 @@ public abstract class BaseMapTest
 
     private static ObjectMapper SHARED_MAPPER;
 
-    protected ObjectMapper objectMapper() {
+    protected ObjectMapper sharedMapper() {
         if (SHARED_MAPPER == null) {
-            SHARED_MAPPER = newObjectMapper();
+            SHARED_MAPPER = newJsonMapper();
         }
         return SHARED_MAPPER;
     }
 
+    protected ObjectMapper objectMapper() {
+        return sharedMapper();
+    }
+
     protected ObjectWriter objectWriter() {
-        return objectMapper().writer();
+        return sharedMapper().writer();
     }
 
     protected ObjectReader objectReader() {
-        return objectMapper().reader();
+        return sharedMapper().reader();
     }
     
     protected ObjectReader objectReader(Class<?> cls) {
-        return objectMapper().readerFor(cls);
+        return sharedMapper().readerFor(cls);
     }
 
-    // @since 2.9
-    protected static ObjectMapper newObjectMapper() {
-        return new ObjectMapper();
+    // @since 2.10
+    protected static ObjectMapper newJsonMapper() {
+        return new JsonMapper();
+    }
+
+    // @since 2.10
+    protected static JsonMapper.Builder jsonMapperBuilder() {
+        return JsonMapper.builder();
     }
 
     // @since 2.7
@@ -237,7 +247,7 @@ public abstract class BaseMapTest
         // this is a work-around; no null modifier added
         return TypeFactory.defaultInstance().withModifier(null);
     }
-    
+
     /*
     /**********************************************************
     /* Additional assert methods
@@ -291,13 +301,13 @@ public abstract class BaseMapTest
     protected String serializeAsString(Object value)
         throws IOException
     {
-        return serializeAsString(objectMapper(), value);
+        return serializeAsString(sharedMapper(), value);
     }
 
     protected String asJSONObjectValueString(Object... args)
         throws IOException
     {
-        return asJSONObjectValueString(objectMapper(), args);
+        return asJSONObjectValueString(sharedMapper(), args);
     }
 
     protected String asJSONObjectValueString(ObjectMapper m, Object... args)
@@ -319,7 +329,7 @@ public abstract class BaseMapTest
     protected <T> T readAndMapFromString(String input, Class<T> cls)
         throws IOException
     {
-        return readAndMapFromString(objectMapper(), input, cls);
+        return readAndMapFromString(sharedMapper(), input, cls);
     }
 
     protected <T> T readAndMapFromString(ObjectMapper m, String input, Class<T> cls) throws IOException
