@@ -9,12 +9,14 @@ import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonTypeInfo;
 import com.fasterxml.jackson.annotation.JsonTypeInfo.As;
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.type.TypeReference;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.ObjectMapper.DefaultTyping;
 import com.fasterxml.jackson.databind.deser.ContextualDeserializer;
 import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
 import com.fasterxml.jackson.databind.deser.std.StdScalarDeserializer;
 import com.fasterxml.jackson.databind.module.SimpleModule;
+import com.fasterxml.jackson.databind.testutil.NoCheckSubTypeValidator;
 
 /**
  * Unit tests for verifying "raw" (or "untyped") data binding from JSON to JDK objects;
@@ -90,7 +92,7 @@ public class UntypedDeserializationTest
         {
             Map<String,Object> map = new LinkedHashMap<String,Object>();
             while (p.nextValue() != JsonToken.END_OBJECT) {
-                map.put(p.getCurrentName(), "Y"+p.getText());
+                map.put(p.currentName(), "Y"+p.getText());
             }
             return map;
         }
@@ -108,19 +110,27 @@ public class UntypedDeserializationTest
     static class WrappedPolymorphicUntyped {
         @JsonTypeInfo(use=JsonTypeInfo.Id.CLASS)
         public Object value;
+
+        protected WrappedPolymorphicUntyped() { }
+        public WrappedPolymorphicUntyped(Object o) { value = o; }
     }
 
     static class WrappedUntyped1460 {
         public Object value;
     }
 
+    // [databind#2115]
+    static class SerContainer {
+        public java.io.Serializable value;
+    }
+
     /*
     /**********************************************************
     /* Test methods
     /**********************************************************
      */
 
-    private final ObjectMapper MAPPER = new ObjectMapper();
+    private final ObjectMapper MAPPER = newJsonMapper();
     
     @SuppressWarnings("unchecked")
     public void testSampleDoc() throws Exception
@@ -199,6 +209,22 @@ public class UntypedDeserializationTest
         assertNull(result.get(3));
     }
 
+    public void testSimpleVanillaScalars() throws IOException
+    {
+        assertEquals("foo", MAPPER.readValue(quote("foo"), Object.class));
+
+        assertEquals(Boolean.TRUE, MAPPER.readValue(" true ", Object.class));
+
+        assertEquals(Integer.valueOf(13), MAPPER.readValue("13 ", Object.class));
+        assertEquals(Double.valueOf(0.5), MAPPER.readValue("0.5 ", Object.class));
+    }
+
+    public void testSimpleVanillaStructured() throws IOException
+    {
+        List<?> list = (List<?>) MAPPER.readValue("[ 1, 2, 3]", Object.class);
+        assertEquals(Integer.valueOf(1), list.get(0));
+    }
+    
     public void testNestedUntypes() throws IOException
     {
         // 05-Apr-2014, tatu: Odd failures if using shared mapper; so work around:
@@ -214,25 +240,6 @@ public class UntypedDeserializationTest
         assertEquals(2, l.size());
         assertEquals(Integer.valueOf(2), l.get(1));
     }
-    
-    // Allow 'upgrade' of big integers into Long, BigInteger
-    public void testObjectSerializeWithLong() throws IOException
-    {
-        final ObjectMapper mapper = new ObjectMapper();
-        mapper.enableDefaultTyping(DefaultTyping.JAVA_LANG_OBJECT, As.PROPERTY);
-        final long VALUE = 1337800584532L;
-
-        String serialized = "{\"timestamp\":"+VALUE+"}";
-        // works fine as node
-        JsonNode deserialized = mapper.readTree(serialized);
-        assertEquals(VALUE, deserialized.get("timestamp").asLong());
-        // and actually should work in Maps too
-        Map<?,?> deserMap = mapper.readValue(serialized, Map.class);
-        Number n = (Number) deserMap.get("timestamp");
-        assertNotNull(n);
-        assertSame(Long.class, n.getClass());
-        assertEquals(Long.valueOf(VALUE), n);
-    }
 
     public void testUntypedWithCustomScalarDesers() throws IOException
     {
@@ -276,39 +283,6 @@ public class UntypedDeserializationTest
         assertEquals(2, l.size());
         assertTrue(l.get(0) instanceof Map<?,?>);
         assertTrue(l.get(1) instanceof List<?>);
-
-        ObjectReader rDefault = mapper.readerFor(WrappedPolymorphicUntyped.class);
-        ObjectReader rAlt = rDefault
-                .with(DeserializationFeature.USE_BIG_DECIMAL_FOR_FLOATS,
-                        DeserializationFeature.USE_BIG_INTEGER_FOR_INTS);
-        WrappedPolymorphicUntyped w;
-
-        w = rDefault.readValue(aposToQuotes("{'value':10}"));
-        assertEquals(Integer.valueOf(10), w.value);
-        w = rAlt.readValue(aposToQuotes("{'value':10}"));
-        assertEquals(BigInteger.TEN, w.value);
-
-        w = rDefault.readValue(aposToQuotes("{'value':5.0}"));
-        assertEquals(Double.valueOf(5.0), w.value);
-        w = rAlt.readValue(aposToQuotes("{'value':5.0}"));
-        assertEquals(new BigDecimal("5.0"), w.value);
-
-        StringBuilder sb = new StringBuilder(100).append("[0");
-        for (int i = 1; i < 100; ++i) {
-            sb.append(", ").append(i);
-        }
-        sb.append("]");
-        final String INT_ARRAY_JSON = sb.toString();
-
-        // First read as-is, no type wrapping
-        Object ob = mapper.readerFor(Object.class)
-                .with(DeserializationFeature.USE_JAVA_ARRAY_FOR_JSON_ARRAY)
-                .readValue(INT_ARRAY_JSON);
-        assertTrue(ob instanceof Object[]);
-        Object[] obs = (Object[]) ob;
-        for (int i = 0; i < 100; ++i) {
-            assertEquals(Integer.valueOf(i), obs[i]);
-        }
     }
 
     public void testUntypedWithListDeser() throws IOException
@@ -366,8 +340,9 @@ public class UntypedDeserializationTest
         assertTrue(ob instanceof List<?>);
 
         // but can change to produce Object[]:
-        MAPPER.configure(DeserializationFeature.USE_JAVA_ARRAY_FOR_JSON_ARRAY, true);
-        ob = MAPPER.readValue("[1]", Object.class);
+        ob = MAPPER.readerFor(Object.class)
+            .with(DeserializationFeature.USE_JAVA_ARRAY_FOR_JSON_ARRAY)
+            .readValue("[1]");
         assertEquals(Object[].class, ob.getClass());
     }
 
@@ -383,4 +358,174 @@ public class UntypedDeserializationTest
                 .readValue(JSON);
         assertEquals(Long.valueOf(3), w.value);
     }
+
+    // [databind#2115]: Consider `java.io.Serializable` as sort of alias of `java.lang.Object`
+    // since all natural target types do implement `Serializable` so assignment works
+    public void testSerializable() throws Exception
+    {
+        final String JSON1 = aposToQuotes("{ 'value' : 123 }");
+        SerContainer cont = MAPPER.readValue(JSON1, SerContainer.class);
+        assertEquals(Integer.valueOf(123), cont.value);
+
+        cont = MAPPER.readValue(aposToQuotes("{ 'value' : true }"), SerContainer.class);
+        assertEquals(Boolean.TRUE, cont.value);
+
+        // But also via Map value, even key
+        Map<?,?> map = MAPPER.readValue(JSON1, new TypeReference<Map<String, Serializable>>() { });
+        assertEquals(1, map.size());
+        assertEquals(Integer.valueOf(123), map.get("value"));
+
+        map = MAPPER.readValue(JSON1, new TypeReference<Map<Serializable, Object>>() { });
+        assertEquals(1, map.size());
+        assertEquals("value", map.keySet().iterator().next());
+    }
+
+    /*
+    /**********************************************************
+    /* Test methods, merging
+    /**********************************************************
+     */
+    
+    public void testValueUpdateVanillaUntyped() throws Exception
+    {
+        Map<String, Object> map = new LinkedHashMap<>();
+        map.put("a", 42);
+
+        ObjectReader r = MAPPER.readerFor(Object.class).withValueToUpdate(map);
+        Object result = r.readValue(aposToQuotes("{'b' : 57}"));
+        assertSame(map, result);
+        assertEquals(2, map.size());
+        assertEquals(Integer.valueOf(57), map.get("b"));
+
+        // Try same with other types, too
+        List<Object> list = new ArrayList<>();
+        list.add(1);
+        r = MAPPER.readerFor(Object.class).withValueToUpdate(list);
+        result = r.readValue("[ 2, true ]");
+        assertSame(list, result);
+        assertEquals(3, list.size());
+        assertEquals(Boolean.TRUE, list.get(2));
+    }
+
+    public void testValueUpdateCustomUntyped() throws Exception
+    {
+        SimpleModule m = new SimpleModule("test-module")
+                .addDeserializer(String.class, new UCStringDeserializer());
+        final ObjectMapper customMapper = newJsonMapper()
+            .registerModule(m);
+        
+        Map<String, Object> map = new LinkedHashMap<>();
+        map.put("a", 42);
+
+        ObjectReader r = customMapper.readerFor(Object.class).withValueToUpdate(map);
+        Object result = r.readValue(aposToQuotes("{'b' : 'value', 'c' : 111222333444, 'enabled':true}"));
+        assertSame(map, result);
+        assertEquals(4, map.size());
+        assertEquals("VALUE", map.get("b"));
+        assertEquals(Long.valueOf(111222333444L), map.get("c"));
+        assertEquals(Boolean.TRUE, map.get("enabled"));
+
+        // Try same with other types, too
+        List<Object> list = new ArrayList<>();
+        list.add(1);
+        r = customMapper.readerFor(Object.class).withValueToUpdate(list);
+        result = r.readValue(aposToQuotes("[ 2, 'foobar' ]"));
+        assertSame(list, result);
+        assertEquals(3, list.size());
+        assertEquals("FOOBAR", list.get(2));
+    }
+    
+    /*
+    /**********************************************************
+    /* Test methods, polymorphic
+    /**********************************************************
+     */
+
+    // Allow 'upgrade' of big integers into Long, BigInteger
+    public void testObjectSerializeWithLong() throws IOException
+    {
+        final ObjectMapper mapper = newJsonMapper();
+        mapper.activateDefaultTyping(NoCheckSubTypeValidator.instance,
+                DefaultTyping.JAVA_LANG_OBJECT, As.PROPERTY);
+        final long VALUE = 1337800584532L;
+
+        String serialized = "{\"timestamp\":"+VALUE+"}";
+        // works fine as node
+        JsonNode deserialized = mapper.readTree(serialized);
+        assertEquals(VALUE, deserialized.get("timestamp").asLong());
+        // and actually should work in Maps too
+        Map<?,?> deserMap = mapper.readValue(serialized, Map.class);
+        Number n = (Number) deserMap.get("timestamp");
+        assertNotNull(n);
+        assertSame(Long.class, n.getClass());
+        assertEquals(Long.valueOf(VALUE), n);
+    }
+
+    public void testPolymorphicUntypedVanilla() throws IOException
+    {
+        ObjectReader rDefault = MAPPER.readerFor(WrappedPolymorphicUntyped.class);
+        ObjectReader rAlt = rDefault
+                .with(DeserializationFeature.USE_BIG_DECIMAL_FOR_FLOATS,
+                        DeserializationFeature.USE_BIG_INTEGER_FOR_INTS);
+        WrappedPolymorphicUntyped w;
+
+        w = rDefault.readValue(aposToQuotes("{'value':10}"));
+        assertEquals(Integer.valueOf(10), w.value);
+        w = rAlt.readValue(aposToQuotes("{'value':10}"));
+        assertEquals(BigInteger.TEN, w.value);
+
+        w = rDefault.readValue(aposToQuotes("{'value':5.0}"));
+        assertEquals(Double.valueOf(5.0), w.value);
+        w = rAlt.readValue(aposToQuotes("{'value':5.0}"));
+        assertEquals(new BigDecimal("5.0"), w.value);
+
+        StringBuilder sb = new StringBuilder(100).append("[0");
+        for (int i = 1; i < 100; ++i) {
+            sb.append(", ").append(i);
+        }
+        sb.append("]");
+        final String INT_ARRAY_JSON = sb.toString();
+
+        // First read as-is, no type wrapping
+        Object ob = MAPPER.readerFor(Object.class)
+                .with(DeserializationFeature.USE_JAVA_ARRAY_FOR_JSON_ARRAY)
+                .readValue(INT_ARRAY_JSON);
+        assertTrue(ob instanceof Object[]);
+        Object[] obs = (Object[]) ob;
+        for (int i = 0; i < 100; ++i) {
+            assertEquals(Integer.valueOf(i), obs[i]);
+        }
+    }
+
+    public void testPolymorphicUntypedCustom() throws IOException
+    {
+        // register module just to override one deserializer, to prevent use of Vanilla deser
+        SimpleModule m = new SimpleModule("test-module")
+                .addDeserializer(String.class, new UCStringDeserializer());
+        final ObjectMapper customMapper = newJsonMapper()
+            .registerModule(m);
+        ObjectReader rDefault = customMapper.readerFor(WrappedPolymorphicUntyped.class);
+
+        WrappedPolymorphicUntyped w = rDefault.readValue(aposToQuotes("{'value':10}"));
+        assertEquals(Integer.valueOf(10), w.value);
+
+        w = rDefault.readValue(aposToQuotes("{'value':9988776655}"));
+        assertEquals(Long.valueOf(9988776655L), w.value);
+        w = rDefault.readValue(aposToQuotes("{'value':0.75}"));
+        assertEquals(Double.valueOf(0.75), w.value);
+
+        w = rDefault.readValue(aposToQuotes("{'value':'abc'}"));
+        assertEquals("ABC", w.value);
+        w = rDefault.readValue(aposToQuotes("{'value':false}"));
+        assertEquals(Boolean.FALSE, w.value);
+        w = rDefault.readValue(aposToQuotes("{'value':null}"));
+        assertNull(w.value);
+
+        // but... actually how about real type id?
+        final Object SHORT = Short.valueOf((short) 3);
+        String json = customMapper.writeValueAsString(new WrappedPolymorphicUntyped(SHORT));
+
+        WrappedPolymorphicUntyped result = rDefault.readValue(json);
+        assertEquals(SHORT, result.value);
+    }
 }
